# -*- python -*-
# $RCSfile: console.py,v $
# $Revision: 1.23 $
# $Author: langer $
# $Date: 2004/12/08 20:45:28 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# A window for interactively querying OOF while it's running
# in GUI mode. 

from common import debug
from common import mainthread
from common import utils
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO.GUI import gfxmenu
from common.IO.GUI import oof_mainiteration
from common.IO.GUI import quit
from common.IO.GUI import subWindow
import code, sys, string, re

import gtk, _gtk

defaultfont = gtk.load_font("-*-fixed-*-*-*-*-*-120-*-*-*-*-iso8859-*")

tabspacing = 4

# Preprocessing for some string stuff.
ordinary_printable = string.digits + string.letters + string.punctuation + " "


# Build the file menu outside of the instance, so it doesn't
# have to be managed when instances come and go.

current_console = None

def _close_console(menuitem):
    global current_console
    current_console.gtk.destroy()
    
_console_menu = oofmenu.OOFMenuItem("Console", secret=1, gui_only=1, no_log=1)
mainmenu.OOF.addItem(_console_menu)

_console_file_menu = oofmenu.OOFMenuItem('File', gui_only=1, no_log=1)
_console_menu.addItem(_console_file_menu)

_console_file_menu.addItem(oofmenu.OOFMenuItem(
    'Close', help="Close the console window.",
    callback=_close_console, no_log=1, gui_only=1, accel='w'))

_console_file_menu.addItem(oofmenu.OOFMenuItem(
    'Quit', help="Exit the OOF application.",
    callback=quit.quit, no_log=1, gui_only=1, accel='q'))


# GUIConsole does not have a conventional "interact" function, but
# instead is event-driven by the GUI.
class GUIConsole(code.InteractiveConsole, subWindow.SubWindow):
    def __init__(self, locals, font=defaultfont):
        global _console_menu
        debug.mainthreadTest()
        code.InteractiveConsole.__init__(self, locals=locals)
        subWindow.SubWindow.__init__(
            self, title="OOF2 Python Console",
            menu=_console_menu)
                                    
        #
        self.history_list = []
        self.history_pos = 0
        self.cursor_pos = 0             # position from start of line
        self.raw = None
        self.raw_result = None
        #
        self.font = font
        #
    
        scroll = gtk.GtkScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.text = gtk.GtkText()
        #
        self.mainbox.add(scroll)
        scroll.add(self.text)
        #
        self.text.set_editable(0)
        self.text.set_word_wrap(1)
        self.text.set_usize(600,150)
        self.text.connect("key-press-event", self.key_press)
        #
        style = self.text.get_style()
        self.fg = style.fg[gtk.STATE_NORMAL]
        self.bg = style.bg[gtk.STATE_NORMAL]
        #
        self.gtk.connect("destroy", self.local_destroy)
        #
        # File emulation attributes.
        self.old_stdout = sys.stdout
        sys.stdout = self
        self.softspace = 0
        self.mode="a"
        #
        self.line = ""
        try:
            self.prompt1=sys.ps1
        except AttributeError:
            self.prompt1=">>> "
        try:
            self.prompt2=sys.ps2
        except AttributeError:
            self.prompt2="... "
        self.p1len = len(self.prompt1)
        self.p2len = len(self.prompt2)

        # Do the banner manually, since we don't have "interact".
        self.textout("OOF2 Console:\n"+self.prompt1)
        self.gtk.show_all()

    def close(self, *args):
        print args
        print "Close!!!"

    def grab_focus(self):
        debug.mainthreadTest()
        self.text.grab_focus()
    
    # Manually process keystrokes.  Wotta pain.  For ascii printable,
    # just print.  For cursor-up or cursor-down, do the history.
    # Does not currently accept text via paste (as in cut-and-)
    def key_press(self, gtkobj, event):
        debug.mainthreadTest()
        global ordinary_printable
        if event.string == "":
            if event.keyval==gtk.GDK.Up:
                self.text.emit_stop_by_name("key_press_event")
                # stop at the top. 
                if self.history_pos == len(self.history_list):
                    pass
                else:
                    self.history_pos += 1
                    line = self.history_list[-self.history_pos]
                    self.cursor_pos = len(line)
                    self.replace_line(line)
            elif event.keyval==gtk.GDK.Down:
                self.text.emit_stop_by_name("key_press_event")
                # stop at the bottom.
                if self.history_pos == 0:
                    pass
                else:
                    self.history_pos -= 1
                    if self.history_pos==0:
                        self.cursor_pos = 0
                        self.replace_line("")
                    else:
                        line = self.history_list[-self.history_pos]
                        self.cursor_pos = len(line)
                        self.replace_line(line)
            elif event.keyval==gtk.GDK.Left:
                self.text.emit_stop_by_name("key_press_event")
                if self.cursor_pos > 0:
                    self.cursor_pos -= 1
                    self.replace_line(self.line)
            elif event.keyval==gtk.GDK.Right:
                self.text.emit_stop_by_name("key_press_event")
                if self.cursor_pos < len(self.line):
                    self.cursor_pos += 1
                    self.replace_line(self.line)
            else:                       # we don't understand this key press
                return gtk.FALSE        # key press not processed
        else:                           # event.string != ""
            if event.string in ordinary_printable:
                line = self.line[:self.cursor_pos] + event.string + \
                       self.line[self.cursor_pos:]
                self.cursor_pos += 1
                self.replace_line(line)
                return gtk.TRUE
            else:
                if event.string=='\r' or event.string=='\n':
                    self.text.backward_delete(len(self.line))
                    self.textout(self.line + '\n')
                    # Clear the line immediately, so there is no pending
                    # data if you go into raw mode while inside the
                    # interpreter. 
                    txt = self.line
                    self.cursor_pos = 0
                    self.history_list.append(txt)
                    self.history_pos=0
                    self.line = ""
                    # In raw mode, just park the accumulated string
                    # and mark yourself as done.  Caller
                    # will handle the return.
                    if self.raw:
                        self.raw = None
                        self.raw_result = txt
                    # Otherwise, push the accumulated string to the
                    # interpreter.
                    else:
                        more = self.push(txt)
                        if more:
                            self.textout(self.prompt2)
                        else:
                            self.textout(self.prompt1)
                # Backspace functionality.  Catches both ctrl-H and
                # ASCII delete.
                elif event.string=="\x08" or event.string=='\x7f':
                    if self.cursor_pos > 0:
                        line = self.line[:self.cursor_pos-1] + \
                               self.line[self.cursor_pos:]
                        self.cursor_pos -= 1
                        self.replace_line(line)
                # Tab -- go forward to the next mod-4 location.
                elif event.string=="\t":
                    self.text.emit_stop_by_name("key_press_event")
                    pad = " "*(tabspacing-self.cursor_pos%tabspacing)
                    line = self.line[:self.cursor_pos] + pad \
                           + self.line[self.cursor_pos:]
                    self.cursor_pos += len(pad)
                    self.replace_line(line)
                    return gtk.TRUE
                elif event.string=='\x0b': # control K kills to end of line
                    line = self.line[:self.cursor_pos]
                    self.replace_line(line)
                elif event.string=='\x01': # control A goes to beginning of line
                    self.cursor_pos = 0
                    self.replace_line(self.line)
                elif event.string=='\x05': # control E goes to end of line
                    self.cursor_pos = len(self.line)
                    self.replace_line(self.line)
                # Allow ctrl-D out of interpreter mode.  Disallow for
                # raw mode, because otherwise the wrapping function
                # will never return.
                elif event.string=="\x04":
                    if not self.raw:
                        self.gtk.destroy()
                else:                   # we don't understand this key press
                    return gtk.FALSE    # key press not processed
        return gtk.TRUE                 # key press was processed

    def replace_line(self, newtext):
        debug.mainthreadTest()
        self.text.freeze()
        old_line = self.line
        self.line = newtext
        line = self.line + ' '          # room at end for possible cursor
        self.text.backward_delete(len(old_line)+1) # extra for cursor.
        self.text.insert(self.font, None, None, line[:self.cursor_pos])
        self.text.insert(self.font, self.bg, self.fg, line[self.cursor_pos])
        self.text.insert(self.font, None, None, line[self.cursor_pos+1:])
        self.text.thaw()
        return old_line

    # Reporter messages generated by commands issued in the Console
    # can come from the wrong thread, so make sure we're in the main
    # thread before writing.
    def textout(self, string):
        mainthread.runBlock(self.textout_thread, (string,))
    def textout_thread(self, string):
        if self.text.get_length()>0:
            self.text.backward_delete(1) # cursor
        self.text.insert(self.font, None, None, string)
        self.text.insert(self.font, self.bg, self.fg, ' ') # cursor

    # Clean up output mechanisms that were hijacked before the init.
    def local_destroy(self, gtkobj):
        global current_console
        sys.stdout = self.old_stdout
        current_console = None
                
    # File-like functionality, allowing the console to stand
    # in for stdout.  Lots of no-ops.
    def close(self):
        pass

    def flush(self):
        pass

    def read(self, size=0):
        raise EOFError

    def readline(self, size=0):
        raise EOFError
    
    def seek(self, offset, whence=0):
        pass

    def tell(self):
        return self.gtk.get_length()

    def write(self, data):
        self.textout(data)

    def writelines(self, lines):
        for l in lines:
            self.write(l)
    
def make_console(menuitem):
    global current_console
    if current_console:
        current_console.raise_window()
    else:
        current_console = GUIConsole(sys.modules['__main__'].__dict__)
    current_console.grab_focus()

def consolation(menuitem):
    print "There, there, I'm sure everything will be fine."

_console_trigger = oofmenu.OOFMenuItem('Console',
                                       callback=consolation,
                                       gui_callback=make_console,
                                       help="Open or raise the Python console interface.",
                                       no_log=1)
    
mainmenu.OOF.Windows.addItem(_console_trigger)

# Operate the console in "raw" mode -- writes out a prompt, then
# returns whatever was typed from then until a new line was
# encountered. 
def raw_input(prompt):
    global current_console
    if current_console:
        current_console.raw=1
        current_console.write(prompt)
        # Newline on input will kick the console out of "raw" mode,
        # that's when we're done.
        while current_console.raw:
            # gtk.mainiteration()
            oof_mainiteration.mainiteration()
        result = current_console.raw_result
        current_console.raw_result=None
        return result
    else:
        return sys.modules['__main__'].__builtins__.raw_input(prompt)


# Over-ride the "raw_input" function as seen from the
# OOF evaluation namespace.
utils.OOFdefine('raw_input', raw_input)
