# -*- python -*-
# $RCSfile: pixelinfo.py,v $
# $Revision: 1.11 $
# $Author: haan $
# $Date: 2004/11/17 22:59:54 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from SWIG.common import timestamp
from common import toolbox
from common import primitives
from common.IO import oofmenu
from common.IO import parameter
from common.IO import reporter

class PixelInfoToolbox(toolbox.Toolbox):
    def __init__(self, gfxwindow):
        self.point = None               # location of last query
        self.timestamp = timestamp.TimeStamp()  # display purpose
        toolbox.Toolbox.__init__(self, "Pixel_Info", gfxwindow)
        
    def makeMenu(self, menu):
        self.menu = menu
        menu.addItem(oofmenu.OOFMenuItem(
            'Query',
            callback=self.queryPixel,
            params=[parameter.IntParameter('x', 0, tip="The x coordinate."),
                    parameter.IntParameter('y', 0, tip="The y coordinate.")],
            help="Query the pixel that is closest to the given point(x,y)."
            ))
    def queryPixel(self, menuitem, x, y): # menu callback
        self.timestamp.increment()
        self.point = primitives.iPoint(x, y)
        switchboard.notify(self)        # caught by GUI toolbox
        switchboard.notify('redraw')

    def currentPixel(self):
        return self.point

    def findMicrostructure(self):
        who = self.gfxwindow().topwho('Microstructure', 'Image',
                                              'Skeleton', 'Mesh')
        if who is not None:
            return who.getMicrostructure()
        return None

    def getTimeStamp(self):
        return self.timestamp
    
toolbox.registerToolboxClass(PixelInfoToolbox, ordering=1.0)
