# -*- python -*-
# $RCSfile: activeareamod.py,v $
# $Revision: 1.17 $
# $Author: haan $
# $Date: 2004/12/09 22:51:58 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from SWIG.common import pixelselectioncourier
from SWIG.common import switchboard
from common import debug
from common import registeredclass
from common.IO import parameter
from common.IO import whoville
from common.IO import pixelgroupparam
from common.IO import xmlmenudump
import common.microstructure

## Selecting pixels in the active area object makes them *inactive*.
## So activate==unselect, deactivate==select.

def modify(menuitem, microstructure, **params):
    registration = menuitem.data
    modifier = registration(**params)
    ms = common.microstructure.microStructures[microstructure].getObject()
    context = ms.activearea
    context.begin_writing()
    modifier(ms)
    context.end_writing()
    switchboard.notify('active area modified', modifier)
    switchboard.notify('redraw')
   

##########################

class ActiveAreaModifier(registeredclass.RegisteredClass):
    registry = []
    def __call__(self):
        pass

##########################

class ActivateAll(ActiveAreaModifier):
    def __call__(self, microstructure):
        microstructure.activearea.start()
        microstructure.activearea.clear()

registeredclass.Registration('Activate All', ActiveAreaModifier,
                             ActivateAll,
                             ordering=0.0,
                             tip="Activate all pixels",
                             discussion="<para>Make the entire domain of the current &micro; active.</para>"
                             )

##########################

GroupSelection = pixelselectioncourier.GroupSelection

class ActivateOnlySelection(ActiveAreaModifier):
    def __call__(self, microstructure):
        activearea = microstructure.activearea
        pixelselection = microstructure.pixelselection
        activearea.start()
        activearea.clear()              # everything is active
        activearea.invert()             # everything is inactive
##         activearea.unselect(pixelselection.getSelection()) # selection is active
        pixgrp = pixelselection.getSelectionAsGroup()
        debug.fmsg('pixgrp.size=', len(pixgrp), pixgrp.__class__.__name__)
        courier = GroupSelection(microstructure, pixgrp)
        debug.fmsg('courier=', courier.repr())
        activearea.unselectWithoutCheck(courier)
##        activearea.unselectWithoutCheck(GroupSelection(microstructure, pixgrp))

registeredclass.Registration('Activate Selection Only', ActiveAreaModifier,
                             ActivateOnlySelection,
                             ordering=0.1,
                             tip="Activate the selected pixels, deactivating everything else.",
                             discussion=xmlmenudump.loadFile('DISCUSSIONS/common/menu-activate_selection_only.xml')
                             )

class ActivateSelection(ActiveAreaModifier):
    def __call__(self, microstructure):
        activearea = microstructure.activearea
        pixelselection = microstructure.pixelselection
        activearea.start()
##         activearea.unselect(pixelselection.getSelection())
        pixgrp = pixelselection.getSelectionAsGroup()
        activearea.unselectWithoutCheck(GroupSelection(microstructure, pixgrp))

registeredclass.Registration('Activate Selection', ActiveAreaModifier,
                             ActivateSelection,
                             ordering=0.5,
                             tip='Activate the the selected pixels.',
                             discussion="<para>Activate the <link linkend='Section:Concepts:Microstructure:PixelSelection'>selected pixels</link> -- add them to the existing &active;.</para>"
                             )

class ActivateOnlyGroup(ActiveAreaModifier):
    def __init__(self, group):
        self.group = group
    def __call__(self, microstructure):
        grp = microstructure.findGroup(self.group)
        if grp:
            activearea = microstructure.activearea
            activearea.start()
            activearea.clear()
            activearea.invert()
##             activearea.unselect(grp.members())
            activearea.unselectWithoutCheck(GroupSelection(microstructure,
                                                                 grp))

registeredclass.Registration('Activate Pixel Group Only', ActiveAreaModifier,
                             ActivateOnlyGroup,
                             ordering=0.6,
                             params=[
    pixelgroupparam.PixelGroupParameter('group', tip="A pixel group to be activated.")],
                             tip="Activate only the pixels in the given pixel group, deactivating everything else.",
                             discussion=xmlmenudump.loadFile('DISCUSSIONS/common/menu-activate_pixel_group_only.xml')
                             )

class ActivateGroup(ActiveAreaModifier):
    def __init__(self, group):
        self.group = group
    def __call__(self, microstructure):
        grp = microstructure.findGroup(self.group)
        if grp:
            activearea = microstructure.activearea
            activearea.start()
##          activearea.unselect(grp.members())
            activearea.unselectWithoutCheck(GroupSelection(microstructure, grp))

registeredclass.Registration('Activate Pixel Group', ActiveAreaModifier,
                             ActivateGroup,
                             ordering=0.7,
                             params=[pixelgroupparam.PixelGroupParameter('group', tip="A pixel group to be activated.")],
                             tip="Activate the pixels in the given pixel group.",
                             discussion="<para>Activate the selected <link linkend ='Section:Concepts:Microstructure:Pixel_Group'>pixel group</link>. This region will be added to the existing &active;.</para>"
                             ) 
##########################

## DeactivateOnlySelection probably isn't really needed.

##class DeactivateOnlySelection(ActiveAreaModifier):
##    def __call__(self, microstructure):
##        activearea = microstructure.activearea
##        pixelselection = microstructure.pixelselection
##        activearea.start()
##        activearea.clear()              # everything is active
##        activearea.select(pixelselection.getSelection())

##registeredclass.Registration('Deactivate Selection Only', ActiveAreaModifier,
##                             DeactivateOnlySelection,
##                             ordering=1.0,
##                             tip='Deactivate the region under the selected pixels, activating everything else.')
        
class DeactivateSelection(ActiveAreaModifier):
    def __call__(self, microstructure):
        activearea = microstructure.activearea
        pixelselection = microstructure.pixelselection
        activearea.start()
##         activearea.select(pixelselection.getSelection())
        pixgrp = pixelselection.getSelectionAsGroup()
        activearea.selectWithoutCheck(GroupSelection(microstructure, pixgrp))
        

registeredclass.Registration('Deactivate Selection', ActiveAreaModifier,
                             DeactivateSelection,
                             ordering=1.5,
                             tip='Deactivate the selected pixels.',
                             discussion=xmlmenudump.loadFile('DISCUSSIONS/common/menu-deactivate_selection.xml')
                             )

class DeactivateGroup(ActiveAreaModifier):
    def __init__(self, group):
        self.group = group
    def __call__(self, microstructure):
        grp = microstructure.findGroup(self.group)
        if grp:
            activearea = microstructure.activearea
            activearea.start()
##             activearea.select(grp.members())
            activearea.selectWithoutCheck(GroupSelection(microstructure, grp))

registeredclass.Registration('Deactivate Group', ActiveAreaModifier,
                             DeactivateGroup,
                             ordering=1.6,
                             params=[
    pixelgroupparam.PixelGroupParameter('group', tip="A pixel group to be deactivated.")],
                             tip='Deactivate the pixels in the given pixel group.',
                             discussion=xmlmenudump.loadFile('DISCUSSIONS/common/menu-deactivate_group.xml')
                             )

############################

ShrinkSelection = pixelselectioncourier.ShrinkSelection
class ExpandActiveArea(ActiveAreaModifier):
    def __init__(self, radius):
        self.radius = radius
    def __call__(self, microstructure):
        activearea = microstructure.activearea
        activearea.start()
        activegrp = activearea.getSelectionAsGroup()
##         newpixels = activegrp.shrink(self.radius)
##         activearea.unselect(newpixels)
        activearea.unselectWithoutCheck(ShrinkSelection(microstructure,
                                                        activegrp,
                                                        self.radius))
                                                        
registeredclass.Registration('Expand', ActiveAreaModifier,
                             ExpandActiveArea,
                             ordering=3.0,
                             params=[parameter.FloatParameter('radius', 1.0, tip="The radius of exapnsion in pixel units.")],
                             tip="Expand the active area to include all pixels within the given distance from the currently active pixels.",
                             discussion="<para>Expansion of an &active;.</para>")

##############################

ExpandSelection = pixelselectioncourier.ExpandSelection
class ShrinkActiveArea(ActiveAreaModifier):
    def __init__(self, radius):
        self.radius = radius
    def __call__(self, microstructure):
        activearea = microstructure.activearea
        activearea.start()
        activegrp = activearea.getSelectionAsGroup()
##         newpixels = activegrp.expand(self.radius)
##         activearea.select(newpixels)
        activearea.selectWithoutCheck(ExpandSelection(microstructure,
                                                      activegrp,
                                                      self.radius))
        
registeredclass.Registration('Shrink', ActiveAreaModifier,
                             ShrinkActiveArea,
                             ordering=3.1,
                             params=[
    parameter.FloatParameter('radius', 1.0, tip="The radius of shrinkage in pixel units.")
    ],
                             tip="Shrink the active area by excluding all pixels within the given distance from the currently inactive pixels.",
                             discussion="<para>Shrinkage of an &active;.</para>")

##############################

class InvertActiveArea(ActiveAreaModifier):
    def __call__(self, microstructure):
        microstructure.activearea.start()
        microstructure.activearea.invert()

registeredclass.Registration('Invert', ActiveAreaModifier,
                             InvertActiveArea,
                             ordering=4.0,
                             tip="Activate the currently inactive pixels, and inactivate the currently active ones.",
                             discussion="<para>Inversion of an &active;.</para>")

##############################

class CopyActiveArea(ActiveAreaModifier):
    def __init__(self, source):
        self.source = source            # source Microstructure
    def __call__(self, microstructure):
        sourceMS = common.microstructure.microStructures[self.source]
        microstructure.activearea.start()
        microstructure.activearea.getObject().setFromGroup(
            sourceMS.getObject().activearea.getSelectionAsGroup())

registeredclass.Registration('Copy', ActiveAreaModifier,
                             CopyActiveArea,
                             ordering=5.0,
                             params=[
    whoville.WhoParameter('source', common.microstructure.microStructures,
                          tip='The source Microstructure of the active area.')],
                             tip="Copy the active pixels from another Microstructure.",
                             discussion="<para>If one has set up a fairly complicated &active; in one &micro; and needs this active area in another Microstructure that one is currently working on, simply copy the active area over.</para>")
