# -*- python -*-
# $RCSfile: boundarybuilderGUI.py,v $
# $Revision: 1.3 $
# $Author: reida $
# $Date: 2004/10/22 20:55:52 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# Smart widgets for the Director, with sufficient context-awareness
# to present only legal options, i.e. if the prospective boundary
# will be a loop, allow clockwise/counterclockwise, otherwise
# only allow left-to-right, top-to-bottom, etc.

from SWIG.common import switchboard
from common import debug
from common import director
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from engine import boundarybuilder
from engine import skeletoncontext
from engine import skeletonsegment
from engine.IO.GUI import skeletongroupwidgets



# DirectorWidget, a smart EnumWidget that actually retrieves the
# requested segment set and sequences it, and then only presents
# allowed options for the Director.  

class DirectorWidget(parameterwidgets.EnumWidget):
    def __init__(self, enumclass, param, scope=None):
        parameterwidgets.EnumWidget.__init__(self, enumclass, param, scope)
        self.skelwidget = scope.findWidget(
            lambda x: isinstance(x, whowidget.WhoWidget)
            and x.whoclass is skeletoncontext.skeletonContexts)
        self.aggwidget = scope.findWidget(
            lambda x: \
            isinstance(x, skeletongroupwidgets.SkeletonAggregateWidget))

        self.segmenter = skeletongroupwidgets.segmenter[
            self.aggwidget.__class__]
            

        # We don't need to worry about the skeleton changing, because we
        # live in a modal dialog box.  So, might as well get the context
        # now.
        self.skelcontext = skeletoncontext.skeletonContexts[
            self.skelwidget.get_value()]
        # Do worry about when the aggregate changes.
        self.agg_callback = switchboard.requestCallback(self.aggwidget,
                                                        self.newAggregate)
        self.update()


    def cleanUp(self):
        switchboard.removeCallback(self.agg_callback)
        parameterwidgets.EnumWidget.cleanUp(self)

    # Called at __init__ time, and also when the aggregate widget changes.
    # Runs loop-check and prevents the selection of disallowed directors.
    # Then runs the enclosed ChooserWidget's update function.
    def update(self):
        debug.mainthreadTest()
        loop = self.loop_check()
        parameterwidgets.EnumWidget.update(self)
        if loop==0:
            inactive_names=director.loopables
        elif loop==1:
            inactive_names=director.unloopables
        else: # Loop is -1, sequencing failed.
            inactive_names=director.loopables+director.unloopables
        
        namelist = [ n for n in list(self.enumclass.names) if
                     n not in inactive_names ]
        if len(namelist)>0:
            self.widget.gtk.set_sensitive(1)
            self.widget.update(namelist, self.enumclass.helpdict)
            self.widgetChanged(1, interactive=0)
        else:
            self.widget.gtk.set_sensitive(0)
            self.widget.update(["No edge sequence"])
            self.widgetChanged(0, interactive=0)
            

    # GTK callback for when a selection is made.
    def selection(self, gtkobj, name):
        parameterwidgets.EnumWidget.selection(self, gtkobj, name)

    # Just need to update when the aggregate changes.
    def newAggregate(self, *args, **kwargs):
        self.update()

    # Returns 1 for loop, 0 for line, -1 if the sequencer fails or if
    # the returned path is of length zero.
    # TODO LATER: The sequenced segments and nodes should be cached
    # somewhere, so the subsequently-called menu item doesn't have to
    # redo all this work.  While the duplication is of course
    # inherently wasteful, segment sequencing is actually pretty fast,
    # so this should be done after other performance enhancements have
    # made it noticeable.
    def loop_check(self):
        seg_set = self.segmenter(self.skelcontext, self.aggwidget.get_value())
        try:
            (segs, nodes) = skeletonsegment.segSequence(seg_set)
        except skeletonsegment.SequenceError:
            return -1
        else:
            if len(nodes)>0:
                if nodes[0]==nodes[-1]:
                    return 1
                return 0
            return -1 # Zero-length list is also invalid.

def _DirectorParameter_makeWidget(self, scope=None):
    return DirectorWidget(self.enumclass, self, scope)

director.DirectorParameter.makeWidget = _DirectorParameter_makeWidget
