# -*- python -*-
# $RCSfile: materialwidget.py,v $
# $Revision: 1.1 $
# $Author: langer $
# $Date: 2004/10/07 21:01:04 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import debug
from common.IO.GUI import chooser
from common.IO.GUI import parameterwidgets
from engine import materialmanager
from engine.IO import materialmenu

class MaterialWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, scope=None):
        self.chooser = chooser.ChooserWidget(materialmanager.getMaterialNames(),
                                             callback=self.chooserCB)
        parameterwidgets.ParameterWidget.__init__(self, self.chooser.gtk, scope)
        self.sbcallbacks = [
            switchboard.requestCallback("new_material", self.update),
            switchboard.requestCallback("remove_material", self.update)
            ]
        if param.value is not None:
            self.set_value(param.value)
        else:
            try:
                self.set_value(materialmanager.getMaterialNames()[0])
            except IndexError:
                self.widgetChanged(validity=0, interactive=0)
    def chooserCB(self, gtkobj, name):
        self.widgetChanged(validity=self.chooser.nChoices()>0, interactive=1)
    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        parameterwidgets.ParameterWidget.cleanUp(self)
    def update(self, *args):
        names = materialmanager.getMaterialNames()
        names.sort()
        debug.fmsg(names)
        self.chooser.update(names)
        self.widgetChanged(len(names) > 0, interactive=0)
    def get_value(self):
        return self.chooser.get_value()
    def set_value(self, material):
        self.chooser.set_state(material)
        self.widgetChanged(validity=(material is not None), interactive=0)

def _MaterialParameter_makeWidget(self, scope=None):
    return MaterialWidget(self, scope)
materialmenu.MaterialParameter.makeWidget = _MaterialParameter_makeWidget
        
class MaterialsWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, scope=None):
        names = materialmanager.getMaterialNames()
        names.sort()
        self.widget = chooser.ScrolledMultiListWidget(names,
                                                      callback=self.widgetCB)
        parameterwidgets.ParameterWidget.__init__(self, self.widget.gtk, scope)
        self.widget.set_selection(param.value)
        self.sbcallbacks = [
            switchboard.requestCallback('new_material', self.newMaterial),
            switchboard.requestCallback('remove_material', self.newMaterial)
            ]
        self.widgetChanged(0, interactive=0) # nothing selected
    def cleanUp(self):
        map(switchboard.removeCallback, self.sbcallbacks)
        parameterwidgets.ParameterWidget.cleanUp(self)
    def newMaterial(self, *args):
        names = materialmanager.getMaterialNames()
        names.sort()
        self.widget.update(names)
    def get_value(self):
        return self.widget.get_value()
    def set_value(self, value):
        self.widget.set_selection(value)
    def widgetCB(self, list):
        self.widgetChanged(len(list) > 0, interactive=1)

def _MaterialsWidget_makeWidget(self, scope=None):
    return MaterialsWidget(self, scope)

materialmenu.ListOfMaterialsParameter.makeWidget = _MaterialsWidget_makeWidget
