# -*- python -*-
# $RCSfile: meshcswidget.py,v $
# $Revision: 1.15 $
# $Author: reida $
# $Date: 2004/12/03 21:47:11 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common.IO import selectionplaceholder
from common.IO import whoville
from common.IO.GUI import chooser
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from engine import mesh
from engine.IO import meshcsparams
from engine.IO import meshmenu

import gtk

# Widget for MeshCrossSectionSet objects.  Just puts up a list of the
# things, and lets the user pick any number of them, or use the
# "selected" one, which has meaning in the context of a mesh object.
# Return value is a (possibly empty) list of strings, or
# selectionplaceholder.selectionPlaceHolder.

class MeshCrossSectionSetParamWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, scope):
        # Find the enclosing mesh widget.
        self.meshwidget = scope.findWidget(
            lambda x: isinstance(x, whowidget.WhoWidget)
            and x.whoclass is mesh.meshes)

        meshname = self.meshwidget.get_value()
        self.meshobj = mesh.meshes[meshname]
        if issubclass(self.meshobj.__class__, whoville.WhoProxy):
            gfxwindow = scope.findData("gfxwindow")
            self.meshobj = mesh.resolve(gfxwindow)

        self.chooser = chooser.MultiListWidget()
        self.selected = gtk.GtkCheckButton("Selected")
        self.selected.connect("clicked", self.selectedCB)

        self.gtk = gtk.GtkFrame()
        vbox = gtk.GtkVBox()
        self.gtk.add(vbox)
        vbox.add(self.selected)
        vbox.add(self.chooser.gtk)
        
        parameterwidgets.ParameterWidget.__init__(self, self.gtk, scope)
        self.set_value(param.value)

        self.gtk.show_all()

    def selectedCB(self, gtkobj):
        if self.selected.get_active():
            self.set_value(selectionplaceholder.selectionPlaceHolder)
        else:
            self.chooser.clear()
            self.chooser.gtk.set_sensitive(1)
            lcsn=self.meshobj.selectedCSName()
            if lcsn:
                self.chooser.set_selection([lcsn])
        self.widgetChanged(1, interactive=1)

    
    def set_value(self, value):
        self.chooser.update(self.meshobj.allCrossSectionNames())
        self.chooser.clear()
        if value==selectionplaceholder.selectionPlaceHolder:
            lcsn=self.meshobj.selectedCSName()
            if lcsn:
                self.chooser.set_selection([lcsn])
            self.chooser.gtk.set_sensitive(0)
            self.selected.set_active(1)
        else:  # Value is list of names.
            self.chooser.gtk.set_sensitive(1)
            self.selected.set_active(0)
            self.chooser.set_selection(value)
        self.widgetChanged(1, interactive=0)
            
    def get_value(self):
        if self.selected.get_active():
            return selectionplaceholder.selectionPlaceHolder
        else:
            return self.chooser.get_value()

def _make_MCSSPWidget(self, scope):
    return MeshCrossSectionSetParamWidget(self, scope)

meshcsparams.MeshCrossSectionSetParameter.makeWidget = _make_MCSSPWidget

###################

# Widget for picking a single cross section from the set of cross
# sections already defined on a mesh.  Also includes a "New..." button
# for defining a new cross section.

class MeshCrossSectionParamWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param=None, scope=None):
        # Find the enclosing mesh widget.
        self.meshwidget = scope.findWidget(
            lambda x: isinstance(x, whowidget.WhoWidget)
            and x.whoclass is mesh.meshes)

        meshname = self.meshwidget.get_value()
        if meshname != "::":
            self.meshobj = mesh.meshes[meshname]
            if issubclass(self.meshobj.__class__, whoville.WhoProxy):
                gfxwindow = scope.findData("gfxwindow")
                self.meshobj = mesh.resolve(gfxwindow)

        self.chooser = chooser.NewChooserWidget([], callback=self.chooserCB,
                                                button_callback=self.newcsCB)

        parameterwidgets.ParameterWidget.__init__(
            self, self.chooser.gtk, scope)

        switchboard.requestCallback(self.meshwidget, self.mesh_update)
        switchboard.requestCallback("cross sections changed",
                                    self.cs_update)
        

        if param:
            self.set_value(param.value)

        self.update(interactive=0)
        self.gtk.show_all()

    def mesh_update(self, interactive):
        self.update(interactive)
    def cs_update(self):
        self.update(interactive=0)
        
    def update(self, interactive):
        meshname = self.meshwidget.get_value()
        if meshname!="::":
            self.meshobj = mesh.meshes[meshname]
            self.chooser.update(self.meshobj.allCrossSectionNames())
            self.chooser.set_state(self.meshobj.selectedCSName())
            self.chooser.set_button_sensitivity(1)
        else:
            self.meshobj = None
            self.chooser.update([])
            self.chooser.set_button_sensitivity(0)
        # This update can affect the validity, of course.
        if self.chooser.get_value():
            self.widgetChanged(1, interactive)
        else:
            self.widgetChanged(0, interactive)
            
    def chooserCB(self, *args):
        self.widgetChanged(self.chooser.nChoices()>0, interactive=1)

    # callback for the "New..." button in the NewChooserWidget
    def newcsCB(self):
        menuitem = meshmenu.csmenu.New
        if parameterwidgets.getParameters(menuitem.get_arg("name"),
                                          menuitem.get_arg("cross_section"),
                                          title="New cross section",
                                          scope=self.scope):
            menuitem.callWithDefaults(mesh=self.meshwidget.get_value())

    def set_value(self, value):
        self.chooser.set_state(value)
        if value:
            self.widgetChanged(1, interactive=0)
        else:
            self.widgetChanged(0, interactive=0)

    def get_value(self):
        return self.chooser.get_value()

    
def _make_MCSPWidget(self, scope):
    return MeshCrossSectionParamWidget(self, scope)

meshcsparams.MeshCrossSectionParameter.makeWidget = _make_MCSPWidget
