# -*- python -*-
# $RCSfile: meshinfoGUI.py,v $
# $Revision: 1.67 $
# $Author: langer $
# $Date: 2004/09/30 21:42:36 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from SWIG.engine import planarity
from common import debug
from common import guitop
from common import mainthread
from common.IO.GUI import chooser
from common.IO.GUI import mousehandler
from common.IO.GUI import toolboxGUI
from engine.IO import meshinfo
from engine.IO.GUI import meshdataGUI
from gtk import *
import engine.mesh
import string
import math
import SWIG.engine.field
import SWIG.engine.flux

xpadding = 3
xoptions = EXPAND|FILL

## TODO LATER: Can SkeletonToolboxGUI and MeshToolboxGUI share a base
## class?

class MeshInfoMode:
    # Base class for ElementMode, NodeMode.
            
    def __init__(self, toolbox):
        debug.mainthreadTest()
        self.toolbox = toolbox  # MeshToolboxGUI
        self.menu = self.toolbox.toolbox.menu  # ie, gfxtoolbox.toolbox.menu

        self.gtk = GtkVBox()
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.gtk.add(scroll)
        self.vbox = GtkVBox()
        scroll.add_with_viewport(self.vbox)
        frame = GtkFrame(self.targetname + " Information")
        self.vbox.add(frame, expand=0, fill=0)
        self.table = GtkTable(rows=self.nrows, cols=self.ncols)
        frame.add(self.table)

    def destroy(self):
        mainthread.runBlock(self.gtk.destroy)
        self.toolbox = None  # break circular references

    def clearQuery(self):
        self.updateNothing()
            
    # Utility routines for constructing GUI components
    
    def labelmaster(self, column, row, labeltext):
        debug.mainthreadTest()
        label = GtkLabel(labeltext)
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)

    def entrymaster(self, column, row, editable=0):
        debug.mainthreadTest()
        entry = GtkEntry()
        entry.set_editable(editable)
        self.table.attach(entry, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return entry

    def getNodeCB(self, node):
        self.toolbox.changeModeWithObject(node, "Node")

    def showNodeCB(self, node):
        if node:
            self.toolbox.toolbox.peeker.assignObject(node, "Node")
        else:
            self.toolbox.toolbox.peeker.removeObject("Node")
        switchboard.notify("redraw")
        
    def menumasterNode(self, column, row, objlist=[]):
        debug.mainthreadTest()
        namelist = [None]*len(objlist)
        for i in range(len(objlist)):
            obj = objlist[i]
            name = "%s %d at (%g, %g)" % \
                   (obj.classname(), obj.index(),
                    obj.position().x, obj.position().y)
            namelist[i] = name
        menu = chooser.FramedChooserListWidget(
            objlist=objlist, displaylist=namelist, callback=self.showNodeCB,
            dbcallback=self.getNodeCB, autoselect=None)
        self.table.attach(menu.gtk, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return menu

######################
    
class ElementMode(MeshInfoMode):
    nrows=4
    ncols=2
    targetname = "Element"
    def __init__(self, toolbox, querier=None):
        MeshInfoMode.__init__(self, toolbox)

        self.labelmaster((0,1), (0,1), 'index=')
        self.index = self.entrymaster((1,2), (0,1))

        self.labelmaster((0,1), (1,2), 'type=')
        self.type = self.entrymaster((1,2), (1,2))

        self.labelmaster((0,1), (2,3), 'nodes=')
        self.nodes = self.menumasterNode((1,2), (2,3))

        self.labelmaster((0,1), (3,4), 'material=')
        self.material = self.entrymaster((1,2), (3,4))

    def menucmd(self):
        return self.menu.QueryElement
            
    def updateSomething(self, container):
        debug.mainthreadTest()
        element = container.object
        femesh = container.context.femesh()
        coord = container.position
        
        self.type.set_text(element.masterelement().name())
        self.index.set_text(`element.get_index()`)

        nlist = [node for node in element.node_iterator()]
        self.nodes.destroy()
        self.nodes = self.menumasterNode((1,2), (2,3), nlist)

        mat = element.material()
        if mat:
            self.material.set_text(mat.name())
        else:
            self.material.set_text("<No material>")

    def updateNothing(self):
        debug.mainthreadTest()
        self.type.set_text("")
        self.index.set_text("")
        
        self.nodes.destroy()
        self.nodes = self.menumasterNode((1,2), (2,3))
        self.material.set_text("")

######################
    
class NodeMode(MeshInfoMode):
    targetname = "Node"
    nrows = 3
    ncols = 2
    def __init__(self, toolbox, querier=None):
        MeshInfoMode.__init__(self, toolbox)

        self.labelmaster((0,1), (0,1), 'index=')
        self.index = self.entrymaster((1,2), (0,1))

        self.labelmaster((0,1), (1,2), 'type=')
        self.type = self.entrymaster((1,2), (1,2))

        self.labelmaster((0,1), (2,3), 'position=')
        self.pos = self.entrymaster((1,2), (2,3))

    def updateSomething(self, container):
        debug.mainthreadTest()
        node = container.object
        femesh = container.context.femesh()
        
        self.index.set_text(`node.index()`)
        self.type.set_text(node.classname())
        self.pos.set_text("(%g, %g)" %
                          (node.position().x, node.position().y))

    def updateNothing(self):
        debug.mainthreadTest()
        self.index.set_text("")
        self.type.set_text("")
        self.pos.set_text("")
        
    def menucmd(self):
        return self.menu.QueryNode


############################################################################

modes = [ElementMode, NodeMode]

############################################################################
############################################################################

class MeshToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, meshinfo):
        toolboxGUI.GfxToolbox.__init__(self, "Mesh Info", meshinfo)
        self.mainbox = GtkVBox()
        self.gtk.add(self.mainbox)

        self.modeclass = ElementMode
        self.modeobj = None
        self.modeobjdict = {}

        hbox = GtkHBox()
        self.mainbox.add(hbox, expand=0, fill=0)
        hbox.add(GtkLabel("Click on an: "), expand=0, fill=0)

        self.modebuttons = []
        self.modebuttondict = {}
        for mode in modes:
            if self.modebuttons:
                button = GtkRadioButton(label=mode.targetname,
                                        group=self.modebuttons[0])
            else:
                button = GtkRadioButton(label=mode.targetname)
            self.modebuttons.append(button)
            guitop.top().tooltip(button,
                               "Show " + mode.targetname + "information")
            hbox.add(button, expand=0, fill=0)
            button.set_active(self.modeclass is mode)
            button.connect('clicked', self.changeModeCB, mode)
            self.modebuttondict[mode.targetname] = button
                        
        self.table = GtkTable(cols=2, rows=3)
        self.mainbox.add(self.table, expand=0, fill=0)

        # Display mouse click coordinates
        label = GtkLabel('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,1, xpadding=5, xoptions=FILL)
        self.xtext = GtkEntry()
        self.xtext.set_editable(0)
        self.table.attach(self.xtext, 1,2, 0,1,
                          xpadding=5, xoptions=EXPAND|FILL)
        label = GtkLabel('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 1,2, xpadding=5, xoptions=FILL)
        self.ytext = GtkEntry()
        self.ytext.set_editable(0)
        self.table.attach(self.ytext, 1,2, 1,2,
                          xpadding=5, xoptions=EXPAND|FILL)
        guitop.top().tooltip(self.xtext, "x coordinate of the mouse click")
        guitop.top().tooltip(self.ytext, "y coordinate of the mouse click")

        # End of clicked point display

        self.mainbox.add(GtkHSeparator(), expand=0, fill=0, padding=2)

        self.infoframe = GtkFrame()
        self.mainbox.add(self.infoframe, expand=1, fill=1)
        self.meshcontext = apply(self.toolbox.gfxwindow().topwho, ('Mesh',))
        self.buildInfoGUI(self.modeclass)
        # End of query data display

        hbox = GtkHBox()
        self.mainbox.add(hbox, expand=0, fill=0, padding=2)
        centerbox = GtkHBox()
        hbox.add(centerbox, expand=1, fill=0)
        self.meshdatabutton = GtkButton('New Data Viewer...')
        centerbox.add(self.meshdatabutton, expand=0, fill=0)
        self.meshdatabutton.connect('clicked', self.meshdataCB)
        guitop.top().tooltip(self.meshdatabutton,
                             "Open a window to display fields, fluxes, and other quantities evaluated at the mouse click location.")

        buttonbox = GtkHBox()
        self.mainbox.add(buttonbox, expand=0, fill=0)
        self.prev = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkArrow(at=ARROW_LEFT))
        lbox.add(GtkLabel(' Prev'))
        guitop.top().tooltip(self.prev, "Go back to the previous object.")
        self.prev.add(lbox)
        self.prev.connect('clicked', self.prevQuery)
        buttonbox.add(self.prev, expand=0, fill=0, padding=2)
        
        self.clear = GtkButton("Clear")
        self.clear.connect('clicked', self.clearQuery)
        guitop.top().tooltip(self.clear, "Clear the current query.")
        buttonbox.add(self.clear, expand=1, fill=1, padding=2)

        self.next = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkLabel('Next '), expand=0, fill=0)
        guitop.top().tooltip(self.next, "Go to the next object.")
        lbox.add(GtkArrow(at=ARROW_RIGHT), expand=0,fill=0)
        self.next.add(lbox)
        self.next.connect('clicked', self.nextQuery)
        buttonbox.add(self.next, expand=0, fill=0, padding=2)

        self.mainbox.show_all()
        self.sensitize()

        self.xposition = None
        self.yposition = None
        
        self.sbcallbacks = [switchboard.requestCallback(
            (self.toolbox.gfxwindow(), "query mesh"), self.updateQuery),
                            switchboard.requestCallback(
            (self.toolbox.gfxwindow(), "layers changed"),
                                        self.newLayers),
                            switchboard.requestCallback(
            "field activated", self.fieldFluxChanged),
                            switchboard.requestCallback(
            "field defined", self.fieldFluxChanged),
                            switchboard.requestCallback(
            "equation activated", self.fieldFluxChanged)
                            ]
        
    def buildInfoGUI(self, modeclass):
        debug.mainthreadTest()
        if self.modeobj and self.modeobj.__class__ is modeclass:
            return
        if self.modeobj:
            self.infoframe.remove(self.modeobj.gtk)
        # Reuse existing modeobj if possible
        try:
            self.modeobj = self.modeobjdict[modeclass]
        except KeyError:
            self.modeobj = self.modeobjdict[modeclass] = \
                           modeclass(self)
        self.infoframe.add(self.modeobj.gtk)

    def updateQuery(self):
        if not self.toolbox.querier:
            return
        # See if there's anything to update
        if self.toolbox.querier.object:
            # Change mode if needed.
            self.handleMode(self.toolbox.querier.targetname)
            querier = self.toolbox.querier
            self.modeobj.updateSomething(querier)
            self.showPosition(querier.position.x, querier.position.y)
        else:
            self.modeobj.updateNothing()
        self.sensitize()

    def newLayers(self, *args):
        self.meshcontext = apply(self.toolbox.gfxwindow().topwho, ('Mesh',))
        if self.meshcontext is not None:
            self.buildInfoGUI(self.modeclass)
            self.mainbox.show_all()
            self.updateQuery()
            self.sensitize()
        else:
            self.modeobj.clearQuery()
            self.sensitize()

    def fieldFluxChanged(self, meshname, *args):
        femesh = engine.mesh.meshes[meshname]
        if self.meshcontext and self.meshcontext is femesh:
            self.buildInfoGUI(self.modeclass)
            self.mainbox.show_all()
            self.updateQuery()
            self.sensitize()

    def sensitize(self):
        debug.mainthreadTest()
        self.clear.set_sensitive(self.clearable())
        self.prev.set_sensitive(self.prev_able())
        self.next.set_sensitive(self.next_able())

    def clearable(self):
        try:
            return self.toolbox.querier.clearable()
        except AttributeError:
            return 0

    def prev_able(self):
        return self.toolbox.prev_able()

    def next_able(self):
        return self.toolbox.next_able()

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.toolbox.resetRecords()
        self.sensitize()

    def close(self):
        if self.modeobj:
            self.modeobj.destroy()
        map(switchboard.removeCallback, self.sbcallbacks)
        toolboxGUI.GfxToolbox.close(self)

    def acceptEvent(self, eventtype):
        return eventtype == 'up'

    def up(self, x, y, shift, ctrl):
        self.modeobj.menucmd()(x=x, y=y)

    def changeMode(self, mode):            
        self.modeclass = mode
        self.buildInfoGUI(mode)
        self.mainbox.show_all()
        self.toolbox.resetPeeker()

        # Automatically query object with a current mouse-click point
        if self.xposition is not None and self.yposition is not None:
            self.modeobj.menucmd()(x=self.xposition, y=self.yposition)
        self.sensitize()

    def changeModeCB(self, button, mode):
        if self.modeclass is not mode:
            self.changeMode(mode)

    def blockAutoQuery(self):
        self.xposition = None
        self.yposition = None

    def changeModeWithObject(self, object, objtype):
        self.blockAutoQuery()
        self.modebuttondict[objtype].set_active(1)
        if objtype == "Element":
            self.modeobj.menucmd()(x=object.center().x,
                                   y=object.center().y)
        elif objtype == "Node":
            self.modeobj.menucmd()(x=object.position().x,
                                   y=object.position().y)
        
    def showPosition(self, x, y):
        debug.mainthreadTest()
        self.xtext.set_text("%-13.6g" % x)
        self.ytext.set_text("%-13.6g" % y)
        self.xposition = x
        self.yposition = y

    def prevQuery(self, gtkobj):
        prev = self.toolbox.records.prev().clone()
        targetname = prev.targetname
        self.handleMode(targetname)
        self.toolbox.querier = prev
        self.toolbox.querysignal()

    def clearQuery(self, gtkobj):
        self.modeobj.clearQuery()
        self.toolbox.clearQuerier()
        
    def nextQuery(self, gtkobj):
        next = self.toolbox.records.next().clone()
        targetname = next.targetname
        self.handleMode(targetname)
        self.toolbox.querier = next
        self.toolbox.querysignal()

    def handleMode(self, targetname):
        for mode in modes:
            if mode.targetname==targetname:
                if self.modeclass is not mode:
                    self.blockAutoQuery()
                    self.modebuttons[modes.index(mode)].set_active(1)
                break

    def meshdataCB(self, button):
        meshdataGUI.openMeshData(self.gfxwindow(), self.toolbox.currentPoint())
                              

def _makeGUI(self):
    return MeshToolboxGUI(self)

meshinfo.MeshInfoToolbox.makeGUI = _makeGUI
