# -*- python -*-
# $RCSfile: skeletonselectmenu.py,v $
# $Revision: 1.14 $
# $Author: haan $
# $Date: 2004/11/18 22:33:54 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# The menu for node-selection modification operations, automatically
# added via switchboard callback.

from SWIG.common import lock
from SWIG.common import switchboard
from engine import skeletonselectionmod
from engine import skeletoncontext
from common.microstructure import getMicrostructure
from common import utils
from common.IO import mainmenu
from common.IO import oofmenu
from common.IO import parameter
from common.IO import whoville
from common.IO import xmlmenudump
import types

topmenu = mainmenu.OOF

nodeselectmenu = oofmenu.OOFMenuItem('NodeSelection', cli_only=1)
topmenu.addItem(nodeselectmenu)

segmentselectmenu = oofmenu.OOFMenuItem('SegmentSelection', cli_only=1)
topmenu.addItem(segmentselectmenu)

elementselectmenu = oofmenu.OOFMenuItem('ElementSelection', cli_only=1)
topmenu.addItem(elementselectmenu)

def _undo(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.undo()
    finally:
        selection.end_writing()
    selection.signal()

def _redo(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.redo()
    finally:
        selection.end_writing()
    selection.signal()

def _clear(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.start() # Clear should be undoable.
        selection.clear()
    finally:
        selection.end_writing()
    selection.signal()

def _invert(menuitem, skeleton):
    skelc = skeletoncontext.skeletonContexts[skeleton]
    selection = getattr(skelc, menuitem.data)
    selection.begin_writing()
    try:
        selection.start() # Invert should be undoable.
        selection.invert()
    finally:
        selection.end_writing()
    selection.signal()



def makeMenu(menu, modifier, selection_name):
    menu.clearMenu()

    undo_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Undo", callback=_undo,
        params = [
        whoville.WhoParameter("skeleton",
                              whoville.getClass('Skeleton'),
                              tip=parameter.emptyTipString)],
        help="Undo the latest selection operation.",
        discussion="<para>No harm no foul.</para>"))
    undo_item.data = selection_name

    redo_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Redo", callback=_redo,
              params = [
        whoville.WhoParameter("skeleton",
                              whoville.getClass('Skeleton'),
                              tip=parameter.emptyTipString)],
        help="Redo the latest undone selection operation.",
        discussion="<para>Change of mind? Just undo the 'Undo'.</para>"))
    redo_item.data = selection_name

    clear_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Clear", callback=_clear,
        params = [
        whoville.WhoParameter("skeleton",
                              whoville.getClass('Skeleton'),
                              tip=parameter.emptyTipString)],
        help="Clear the current selection.",
        discussion="<para>Blocking the view? Clear them.</para>"))
    clear_item.data = selection_name

    invert_item = menu.addItem(
        oofmenu.OOFMenuItem(
        "Invert", callback=_invert,
        params = [
        whoville.WhoParameter("skeleton",
                              whoville.getClass('Skeleton'),
                              tip=parameter.emptyTipString)],
        help="Invert the current selection.",
        discussion="<para>When trying to select majority of object, first, select unwanted objects and invert the selection to select the wanted ones.</para>"))
    invert_item.data = selection_name    
        
    for r in modifier.registry:
        # For help
        try:
            help = r.tip
        except AttributeError:
            help = None
        # For discussion
        try:
            discussion = r.discussion
        except AttributeError:
            discussion = None
        menuitem = menu.addItem(
            oofmenu.OOFMenuItem(utils.space2underscore(r.name()),
                                callback = skeletonselectionmod.modify,
                                params = [
            whoville.WhoParameter("skeleton",
                                  whoville.getClass('Skeleton'),
                                  tip=parameter.emptyTipString)] +
                                r.params,
                                help=help,
                                discussion=discussion))
        menuitem.data = r

# Callback for new registration entries added in later imports.
switchboard.requestCallback(skeletonselectionmod.NodeSelectionModifier,
                            makeMenu, nodeselectmenu,
                            skeletonselectionmod.NodeSelectionModifier,
                            "nodeselection")

switchboard.requestCallback(skeletonselectionmod.SegmentSelectionModifier,
                            makeMenu, segmentselectmenu,
                            skeletonselectionmod.SegmentSelectionModifier,
                            "segmentselection")

switchboard.requestCallback(skeletonselectionmod.ElementSelectionModifier,
                            makeMenu, elementselectmenu,
                            skeletonselectionmod.ElementSelectionModifier,
                            "elementselection")


# Insert menu items for the ones already in the registry at
# import-time.
makeMenu(nodeselectmenu, skeletonselectionmod.NodeSelectionModifier,
         "nodeselection")

makeMenu(segmentselectmenu,
         skeletonselectionmod.SegmentSelectionModifier,
         "segmentselection")

makeMenu(elementselectmenu,
         skeletonselectionmod.ElementSelectionModifier,
         "elementselection")
