# -*- python -*-
# $RCSfile: cstrain_.py,v $
# $Revision: 1.4 $
# $Author: langer $
# $Date: 2004/11/18 15:44:23 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.engine import fieldindex
from SWIG.engine import symmmatrix
from SWIG.engine.IO import propertyoutput
from common import debug
from common import enum
from common import registeredclass
from common.IO import parameter

class StrainType(registeredclass.RegisteredClass):
    registry = []
    def name(self):
        return self._name
    def iterator(self, planarity):
        return fieldindex.getSymTensorIterator(planarity)
    def newOutputValue(self):
        return symmmatrix.newSymTensorOutputValue()
 
class GeometricStrain(StrainType):
    _name="Geometric"

registeredclass.Registration(
    "Geometric Strain",
    StrainType,
    GeometricStrain,
    ordering=0,
    tip="The symmetric derivative of the displacement field.",
    discussion="""
    <para> The geometric strain is the symmetric derivative of the
    <link linkend='Section:Concepts:Displacement'>Displacement</link>
    field.  It depends only on the field, and not on any material
    properties.  </para>
    """)

class ElasticStrain(StrainType):
    _name="Elastic"

registeredclass.Registration(
    "Elastic Strain",
    StrainType,
    ElasticStrain,
    ordering=1,
    tip="The part of the geometric strain that contributes to stress.",
    discussion="""
    <para> The elastic strain is the part of the <link
    linkend='RegisteredClass:GeometricStrain'>geometric strain</link>
    that multiplies the elastic modulus when computing the stress.
    For example, in a system with only elasticity and thermal
    expansion, the elastic strain is the geometric strain minus the
    <link linkend='RegisteredClass:ThermalStrain'>thermal
    strain</link>. </para>
    """)

class ThermalStrain(StrainType):
    _name="Thermal"

registeredclass.Registration(
    "Thermal Strain",
    StrainType,
    ThermalStrain,
    ordering=2,
    tip="Strain induced by thermal expansion",
    discussion="""
    <para> The thermal strain is the <link
    linkend='MenuItem:OOF.Property.Parametrize.Thermal.ThermalExpansion.Isotropic'>
    thermal expansion coefficient</link> times the
    temperature. </para>
    """)

class StrainTypeParameter(parameter.RegisteredParameter):
    def __init__(self, name, value=None, default=None, tip=None):
        parameter.RegisteredParameter.__init__(self, name, StrainType,
                                               value, default, tip)
    def clone(self):
        return StrainTypeParameter(self.name, self.value, self.default,
                                   self.tip)

propertyoutput.SymmMatrix3PropertyOutputRegistration(
    "Strain",
    initializer=POInitGeometricStrain(),
    parameters=StrainTypeParameter("type", default=GeometricStrain(),
                                   tip='The type of strain to compute.'),
    tip='Compute a strain.',
    discussion="""

    <para>Compute some variety of strain on the &mesh;.  The parameter
    <varname>type</varname> determines exactly which strain is
    computed.</para>
    
    """)
