# This file was created automatically by SWIG.
import solverstepperc
class SolverStepperCorePtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def nrows(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_nrows,(self,) + _args, _kwargs)
        return val
    def ncols(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_ncols,(self,) + _args, _kwargs)
        return val
    def precompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_precompute,(self,) + _args, _kwargs)
        return val
    def solve(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_solve,(self,) + _args, _kwargs)
        return val
    def postcompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_postcompute,(self,) + _args, _kwargs)
        return val
    def residual(self, *_args, **_kwargs):
        val = apply(solverstepperc.SolverStepperCore_residual,(self,) + _args, _kwargs)
        return val
    def __del__(self,solverstepperc=solverstepperc):
        if self.thisown == 1 :
            solverstepperc.delete_SolverStepperCore(self)
    def __repr__(self):
        return "<C SolverStepperCore instance at %s>" % (self.this,)
class SolverStepperCore(SolverStepperCorePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(solverstepperc.new_SolverStepperCore,_args,_kwargs)
        self.thisown = 1




class NewtonSolverCorePtr(SolverStepperCorePtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def precompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_precompute,(self,) + _args, _kwargs)
        return val
    def solve(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_solve,(self,) + _args, _kwargs)
        return val
    def postcompute(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_postcompute,(self,) + _args, _kwargs)
        return val
    def residual(self, *_args, **_kwargs):
        val = apply(solverstepperc.NewtonSolverCore_residual,(self,) + _args, _kwargs)
        return val
    def __del__(self,solverstepperc=solverstepperc):
        if self.thisown == 1 :
            solverstepperc.delete_NewtonSolverCore(self)
    def __repr__(self):
        return "<C NewtonSolverCore instance at %s>" % (self.this,)
class NewtonSolverCore(NewtonSolverCorePtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(solverstepperc.new_NewtonSolverCore,_args,_kwargs)
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------



#-------------- VARIABLE WRAPPERS ------------------



#-------------- USER INCLUDE -----------------------

# -*- python -*-
# $RCSfile: solverstepper_.py,v $
# $Revision: 1.61 $
# $Author: reida $
# $Date: 2004/12/15 20:44:39 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

import string
from common import debug
from common import cregisteredclass
from common import registeredclass
from common.IO import parameter
from common.IO import reporter
from common.IO import whoville
from SWIG.engine import conjugate
from SWIG.common import switchboard
from SWIG.common import devel
from SWIG.common.IO import progressbar
from SWIG.engine import ooferror2
from SWIG.engine import solver

import engine.mesh
import time

##
## Iterator Managers for SolverDrivers start HERE
##
## A SolverIteratorManager is a class that keeps
## track of the state of the iterations and the
## used solver. If the linear solver used fails
## or the maximum number of iterations is reached
## the SolverIterationManager raises a flag that
## the is used by the SolverDriver to bail-out of
## the loop.

class SolverIterationManager : 
    registry = []
    def __init__(self, num_steps):
        self.max_num_steps = num_steps
        self.continew = 1
        ## self.continuw is a flag to stop
        ## the iterator from doing stuff
        ## 1==keep rocking,
        ## 0==go catch the NIST shuttle
        self.counter = 0 ## number of steps
    def step_number(self):
        return self.counter
    
    def update(self):
        ## update is meant to check the state of the
        ## iterator at the end of each step and
        ## set the most recent value for the state
        ## of the control variables
        pbar = progressbar.getProgress()
        self.counter +=1
        if self.counter >= self.max_num_steps \
               or pbar.query_stop() !=0 \
               or pbar.get_success() is progressbar.FAILED:
            ## If the maximum number of iterations is reached
            ## or somewhere inside the stop button is pushed
            ## or if the state of the progress bar reports fail
            ## then break-out of the loop
            self.continew = 0 ## do not continue state
            pbar.set_message("Failed")
        else:
            ## otherwise, update the state of the progress bar
            ## to reflect the current iteration.
            pbar.set_progress(1.0*self.counter/self.max_num_steps)
            pbar.set_message("%d/%d"%(self.counter, self.max_num_steps))
            
    def checkpoint(self):
        ## return current state of the solver
        return self.continew
    def set_residual(self, residual):
        ## set residual of applied solver
        self.residual = residual
    def get_residual (self):
        ## return value of residual
        return self.residual

## The NewtonIterationManager is a special type of Iteration
## manager that adds extra conditionals to the base
## SolverIterationManager class. Here, the residual of the
## calculation is being tracked. The residual of the linear
## solver only informs about the convergence of the
## increment of the solution, not the total solution.
## 
class NewtonIterationManager (SolverIterationManager):
    def __init__(self, num_steps, tolerance):
        SolverIterationManager.__init__(self, num_steps)
        self.tolerance = tolerance
        self.residual = 100000.0 ## residual is updated on every pass
    def update(self):
        pbar = progressbar.getProgress()
        SolverIterationManager.update(self) ## first check previous flags
        if self.residual <= self.tolerance:
            ## if residual is smaller than specified tolerance
            ## then stop iterating and say that you have succeeded
            self.continew =0
            pbar.set_message("Succeded")
            
## Iterator Managers for SolverDrivers END HERE










################################################
##                                            ##
################################################

## SolverDrivers START HERE

            
## SolverDriver is the base class for all the Solver Iterators
## A linear solver is an element of the solution iteration process.
##
##
## The API is such that self.apply() is the function that has to be called
## to make the SolverDrivers execute the correct work.


class SolverDriver (registeredclass.RegisteredClass) :
    registry = []
    def __init__(self, method, mesh, criterion):
        ## mesh is a who object
        self.method = method ## linear solver that will be used
        self.mesh = mesh ## a who object mesh
        self.core = SolverStepperCore(self.method, mesh.getObject())
        self.criterion = criterion ## a SolverIteratorManager object
        self.status = None ## state of the linear solver at each step
    
    def precompute(self, verbosity = 1):
        ##
        ## if verbosity is 1, then the progress bar mechanisms
        ## of this member function are activated and pbar messages
        ## will be displayed. In any other case, this function
        ## will remain pbar-silent
        ## 
        ## Here, the finite element mesh is built: stiffnes matrix,
        ## boundary conditions are assigned, floating boundary nodes
        ## are promoted, dofs are mapped, and the actual A, x, and b objects
        ## are created (for solving the A x = b)
        ##
        ## first check for quality of materials
        badmatls = self.mesh.getObject().check_materials()
        if badmatls:
            msgs = []
            for matl in badmatls:
                msgs += matl.consistency_messages()
            raise ooferror2.ErrUserError('Badly defined materials!\n' +
                                         string.join(msgs, '\n'))
        ## grab the associated progress object
        eq_Progress = progressbar.getProgress()

        # precompute must be called before anything else that
        # might want to check if a Property is active.
        self.mesh.getObject().precompute() ## precomputes materials
        if verbosity:
            reporter.start_progressbar()
        # smtstart = time.time()

        ## print conjugate.listofconjugatepairs.find_local_mapping(self.mesh.getObject())
        ## self.mesh.getObject().initialize_symmetrize(conjugate.listofconjugatepairs.find_mapping(self.mesh.getObject()))
        ## MASTER STIFFNESS MATRIX IS CONSTRUCTED HERE
        self.mesh.make_stiffness()
        ## makes stiffness matrix and rhs
        ## If equations and fields are propertly defined
        ## each material parameter successfully contributes
        ## to the stiffness matrix and RHS
        
        # smtend = time.time()
        if verbosity:
            reporter.progressbar_completed()
            reporter.end_progressbar()

    ## If process failed equilibration sequence is aborted
        if eq_Progress.query_stop() !=0:
            eq_Progress.set_failure()
            eq_Progress.set_progress(1)
            eq_Progress.set_message("Interrupted") ## GUI message
            if verbosity:
                reporter.progressbar_cleanup()
                reporter.warn("Process interrupted")
            return
    
        
        # Initialize the vector that records the boundary's contribution
        # to the RHS.  Correct size, full of zeros.
        mtx = self.mesh.getObject().stiffness_matrix()
        self.mesh.getObject().boundary_rhs_init( mtx.nrows() )

        
        
        # reporter.report("Construction time: %f" % (smtend-smtstart))
        
        # Make sure all dofs and equations are initially free/independent.
        self.mesh.getObject().resetdofeqs()
        
        ## Resolve intersecting float/fixed BC's, and check fixed BC's.
        ##
        ## Note to developers:
        ## 
        ## Intersecting BC means that more than one boundary condition
        ## is specified at the boundary nodes (i.e., the intersection of
        ## two sets of nodes with user-specified BCs is not null, but
        ## not necessarily redundant).
        ##
        ## A floating boundary condition is one where only the difference
        ## of the values between neighboring nodes is specified. The
        ## field value is not fixed at the node, except when the node
        ## intersects a BC which is fixed (in which case it is promoted).
        ##
        ## A fixed BC is that one where the nodal value of the field
        ## is specified.
        ##
        ## A promoted BC node/dof is a node that is changed in status
        ## from floating to fixed.
        ##
        ## A BC is promoted if it is initially float, but made fixed
        ## if it "intersects" in at least one node with a fixed dof,
        ## thus, directly specifying (fixing) the BC.

        # Explore beautiful British Columbia!!
        while self.mesh.getObject().bcPromote(): 
            pass
        
        
        self.mesh.getObject().fixedBCConsistency()
        if debug.debug() and verbosity:
            reporter.report("Completed Fixed BC consistency checks.")
        
        self.mesh.getObject().floatBCResolve()
        if debug.debug() and verbosity:
            reporter.report("Resolved floating BC intersections, if any.")
        
        self.mesh.getObject().invoke_flux_bcs()
        if debug.debug() and verbosity:
            reporter.report("Applied flux boundary conditions, if any.")
        self.mesh.getObject().invoke_fixed_bcs()
        if debug.debug() and verbosity:
            reporter.report("Applied fixed boundary conditions, if any.")
        self.mesh.getObject().invoke_force_bcs()
        if debug.debug() and verbosity:
            reporter.report("Applied force boundary conditions, if any.")

        self.mesh.getObject().mapdofeqs()      # must come after fixed bcs, before floating ones

        if debug.debug() and verbosity:
            reporter.report("Mapped equations.")
        self.mesh.getObject().invoke_float_bcs()
        if debug.debug() and verbosity:
            reporter.report("Applied floating boundary conditions, if any.")

        ## The next line is the very last thing that must be done
        ## before actually solving the mesh.  It allocates memory and
        ## builds the actual A-matrix, b-vector, x-vector to solve the
        ## system Ax=b
        debug.fmsg("before core.precompute()")
        self.core.precompute()
        debug.fmsg("after core.precompute()")

        ## report stiffness matrix size
        if verbosity:
            rows_A_matrix = self.core.nrows()
            cols_A_matrix = self.core.ncols()
            reporter.report("Made %dx%d stiffness matrix" % (rows_A_matrix,
                                                         cols_A_matrix))
    def iterate(self, verbosity = 1):
        ## iterate is usually called by the
        ## subclasses' apply() function
        while self.criterion.checkpoint():
            self.equilibrate(verbosity) ## calls the solver and manages pbar object
            self.criterion.set_residual(self.core.residual()) ## updates residual value in iterator
            self.criterion.update() ## updates criterion to keep looping
        
    def equilibrate(self, verbosity = 1):
        if verbosity:
            reporter.start_progressbar()
        # solvestart = time.time()
        self.status = self.core.solve()
        # solveend = time.time()
        if verbosity:
            reporter.progressbar_completed()
            reporter.end_progressbar()
            
    def postcompute(self, verbosity = 1):
        ##
        ## Here, floating BCs are expanded, reset,
        ## and history-dependent fields are fixated
        ##
        eq_Progress = progressbar.getProgress()
        if self.status:
            self.mesh.getObject().expand_float_bcs()
            if debug.debug() and verbosity:
                reporter.report("Expanded floating BC's, if any.")
            self.mesh.solutiontimestamp.increment() # Increment the right timestamp

            # Earliest opportunity to remove automatically-generated fixed
            # BCs caused by promotion.
        self.mesh.getObject().reset_bcs() ## not too expensive
        if verbosity:
            reporter.start_progressbar()
        self.mesh.getObject().post_process() ## plasticity fixation and memory effects are set here
        if verbosity:
            reporter.end_progressbar()
        if debug.debug() and verbosity:
            reporter.report("Completed post-processing, if any.")
        if verbosity:
            reporter.progressbar_cleanup()
        if not eq_Progress.get_success() or \
               not eq_Progress.query_stop():
            info = self.method.message()
            if verbosity:
                reporter.report(info)
                
        
    def redraw(self):
        switchboard.notify('redraw')
    

    def update_solver_state(self):
        ## synchronizes the state of the progress object
        ## and what will be displayed in the Solver GUI page
        pbarobj = progressbar.getProgress()
        message = self.method.message()
        ## If you made it this far without failing
        ## it means you succeeded, even if you press
        ## the stop button now
        if pbarobj.get_success() == progressbar.INCOMPLETE:
            pbarobj.set_success()
            pbarobj.set_message("Succeeded")
            self.mesh.setSolverStatus('completed', self.method, message)
        else:
            if pbarobj.query_stop():
                self.mesh.setSolverStatus('interrupted', self.method, message) ## GUI Page message
            else:
                self.mesh.setSolverStatus('failed', self.method, message) ## GUI Page message
            pbarobj.set_failure()
            pbarobj.set_message("Failed")
            self.mesh.setSolverStatus('failed', self.method, 'solver failed')
        switchboard.notify('mesh solved', self.mesh)
        self.redraw() ## redraw ALWAYS after notifying that the mesh has been solved
        
SolverDriver.tip = "Solver iteration manager."
SolverDriver.discussion = "<para>TODO: Add discussion.</para>"

## LinearSolver is a single-step linear solver.
## It is basically a wrapper for the solver(s).
## It is meant to provide a uniform interface to the
## linear solver page.

## TODO WAY WAY LATER: Make the Solvers aware of the equations that
## they should (and can) solve.
        
class LinearSolver (SolverDriver):
    def __init__(self, method):
        self.method = method
    def apply(self, meshcontext, verbosity = 1):
        from SWIG.engine import solverstepper
        equilibrate = SolverDriver(
            self.method, meshcontext, SolverIterationManager(1))
        equilibrate.precompute(verbosity)
        equilibrate.iterate(verbosity)
        equilibrate.postcompute(verbosity)
        equilibrate.update_solver_state() 
        
registeredclass.Registration(
    'LinearSolver',
    SolverDriver, ## superclass
    LinearSolver, ## subclass
    ordering = 100,
    params = [
    parameter.Parameter('method', solver.SolverPtr,
                        tip=parameter.emptyTipString)],
    tip="Single-step linear solver."
    )


##
## NewtonSolver is a Newton-Raphson iterator.
## It is meant to provide a general framework for
## the solution of time-independent, field-dependent
## problems.
##
##
class NewtonSolver (SolverDriver):
    def __init__(self, method, newton_tolerance, newton_max_iterations):
        self.method = method
        self.mesh = None ## mesh is a who object
        self.core = None
        self.newton_tolerance = newton_tolerance
        self.newton_max_iterations = newton_max_iterations
        self.criterion = NewtonIterationManager(self.newton_max_iterations, self.newton_tolerance)
        self.status = None ## state of the linear solver at each step
    def iterate(self, verbosity = 1):
        linear_verbosity = 0 ## internal components are silent
        while self.criterion.checkpoint():
            self.precompute(linear_verbosity)
            self.equilibrate(linear_verbosity)
            self.criterion.set_residual(self.core.residual()) ## updates residual value in iterator
            self.postcompute(linear_verbosity)
            ## print "residual=", self.criterion.residual
            self.criterion.update()
            self.mesh.solutiontimestamp.increment()
        if verbosity:
            nr_residual = self.criterion.get_residual()
            num_steps = self.criterion.step_number()
            if nr_residual < self.newton_tolerance:
                reporter.report("Newton method converged!")
            else:
                reporter.report("Newton method failed.")
            reporter.report("Newton method residual = %f number of completed iterations = %d "%( nr_residual, num_steps))

            ## debug.fmsg("counter=", self.criterion.counter)

    def apply(self, meshcontext, verbosity = 1):
        ## debug.fmsg("entering NR.apply()")
        pbarobj = progressbar.getProgress()
        
        self.mesh = meshcontext
        self.core = NewtonSolverCore(self.method, meshcontext.getObject())
        
        self.iterate(verbosity) ## iterate updates properties at the end of each cycle
        self.update_solver_state()
        
        
        
if devel.devel():
    registeredclass.Registration(
        'NewtonRaphson',
        SolverDriver,
        NewtonSolver,
        ordering=200,
        params=[
        parameter.Parameter('method', solver.SolverPtr,
                            tip='linear solver is used by NR method to find non-linear solution'),
        parameter.FloatParameter('newton_tolerance', 1e-8,
                                 tip='convergence criteria'),
        parameter.IntParameter('newton_max_iterations', 10,
                               tip='maximum number of Newton steps before bailing-out')
        
        ],
        tip='Newton-Raphson solver for the solution of field-dependent material properties'
        )
